// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"flag"
	"strings"
)

func (s *Settings[C]) assignValues(c cmd[C]) {
	flgs := c.flagSet
	flgs.Visit(func(f *flag.Flag) {

		name := f.Name
		value := f.Value.String()

		k, ok := c.tagMapping[name]
		if !ok {
			s.errors = append(s.errors, newUnknownFlagError(name))
			return
		}

		pa := append(c.valuePath, k)
		p := strings.Join(pa, ".")

		err := setValueUsingPath(&s.definitions, p, value)
		if err != nil {
			s.errors = append(s.errors, err)
		}

		err = c.setShadowValue(s.shadow, k, value)
		if err != nil {
			s.errors = append(s.errors, err)
		}

		return

	})

}

func (c cmd[C]) setShadowValue(obj any, k string, value string) error {

	if obj == nil {
		return nil
	}

	// set shadow
	n, ok := c.shadowMapping[k]
	if !ok {
		return nil
	}

	return setValueUsingPath(obj, n, value)
}
